@@ -1,15 +1,26 @@
 #!/usr/bin/python
 import hashlib, re, sys, os, base64, time, random, hmac
+import ripemd
 
-### Elliptic curve parameters
+### Elliptic curve parameters (secp256k1)
 
 P = 2**256-2**32-2**9-2**8-2**7-2**6-2**4-1
 N = 115792089237316195423570985008687907852837564279074904382605163141518161494337
 A = 0
+B = 7
 Gx = 55066263022277343669578718895168534326250603453777594175500187360389116729240
 Gy = 32670510020758816978083085130507043184471273380659243275938904335757337482424
 G = (Gx,Gy)
 
+
+def change_curve(p, n, a, b, gx, gy):
+    global P, N, A, B, Gx, Gy, G
+    P, N, A, B, Gx, Gy = p, n, a, b, gx, gy
+    G = (Gx, Gy)
+
+def getG():
+    return G
+
 ### Extended Euclidean Algorithm
 
 def inv(a,n):
@@ -27,7 +38,7 @@ def get_code_string(base):
     if base == 2: return '01'
     elif base == 10: return '0123456789'
     elif base == 16: return '0123456789abcdef'
-    elif base == 32: return 'abcdefghijklmnopqrstuvwxyz2345657'
+    elif base == 32: return 'abcdefghijklmnopqrstuvwxyz234567'
     elif base == 58: return '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
     elif base == 256: return ''.join([chr(x) for x in range(256)])
     else: raise ValueError("Invalid base!")
@@ -57,7 +68,7 @@ def decode(string,base):
     return result
 
 def changebase(string,frm,to,minlen=0):
-    if frm == to: return lpad(string,minlen)
+    if frm == to: return lpad(string,get_code_string(frm)[0],minlen)
     return encode(decode(string,frm),to,minlen)
 
 ### JSON access (for pybtctool convenience)
@@ -142,7 +153,7 @@ def decode_pubkey(pub,formt=None):
     elif formt == 'bin': return (decode(pub[1:33],256),decode(pub[33:65],256))
     elif formt == 'bin_compressed':
         x = decode(pub[1:33],256)
-        beta = pow(x*x*x+7,(P+1)/4,P)
+        beta = pow(x*x*x+A*x+B,(P+1)/4,P)
         y = (P-beta) if ((beta + ord(pub[0])) % 2) else beta
         return (x,y)
     elif formt == 'hex': return (decode(pub[2:66],16),decode(pub[66:130],16))
@@ -205,7 +216,7 @@ def multiply(pubkey,privkey):
   f1,f2 = get_pubkey_format(pubkey), get_privkey_format(privkey)
   pubkey, privkey = decode_pubkey(pubkey,f1), decode_privkey(privkey,f2)
   # http://safecurves.cr.yp.to/twist.html
-  if not isinf(pubkey) and (pubkey[0]**3+7-pubkey[1]*pubkey[1]) % P != 0: 
+  if not isinf(pubkey) and (pubkey[0]**3+B-pubkey[1]*pubkey[1]) % P != 0: 
       raise Exception("Point not on curve")
   return encode_pubkey(base10_multiply(pubkey,privkey),f1)
 
@@ -267,7 +278,12 @@ def subtract_privkeys(p1, p2):
 
 def bin_hash160(string):
    intermed = hashlib.sha256(string).digest()
-   return hashlib.new('ripemd160',intermed).digest()
+   digest = ''
+   try:
+       digest = hashlib.new('ripemd160',intermed).digest()
+   except:
+       digest = ripemd.RIPEMD160(intermed).digest()
+   return digest
 def hash160(string):
     return bin_hash160(string).encode('hex')
 
@@ -401,7 +417,7 @@ def ecdsa_raw_recover(msghash,vrs):
     v,r,s = vrs
 
     x = r
-    beta = pow(x*x*x+7,(P+1)/4,P)
+    beta = pow(x*x*x+A*x+B,(P+1)/4,P)
     y = beta if v%2 ^ beta%2 else (P - beta)
     z = hash_to_int(msghash)
 
